/* ALPHADVISOR-Indicators - MT4 Custom Indicator
 * Copyright (C) 2017 ALPHADVISOR.
 * 
 *  This file is part of ALPHADVISOR-Indicators.
 *
 *  ALPHADVISOR-Indicators is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  ALPHADVISOR-Indicators is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ALPHADVISOR-Indicators.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  This software is an improved version based on:
 *  TRAIDE-Indicators - MT4 Custom Indicator
 *  Copyright (C) 2015 Inovance, Inc. http://inovancetech.com
 */
#property copyright "Copyright 2017, ALPHADVISOR"
#property link      "http://alphadvisor.com"
#property version   "2.1.0"
#property strict
#property indicator_separate_window

enum TRAIDE_MA_TYPE {
    SMA,   // Simple Moving Average (SMA)
    EMA,   // Exponential Moving Average (EMA)
    WMA,   // Weighted Moving Average (EMA)
    DEMA,  // Double Exponential Moving Average (DEMA)
    TEMA,  // Triple Exponential Moving Average (TEMA)
    TRIMA, // Triangular Moving Average (TRIMA)
    KAMA,  // Kaufman Moving Average (KAMA)
    MAMA,  // MESA Adaptative Moving Average (MAMA)
    T3     // Triple Exponential Moving Average (T3)
};
enum TRAIDE_PRICE_TYPE {
    OPEN_PRICE=1,
    HIGH_PRICE=2,
    LOW_PRICE=3,
    CLOSE_PRICE=4,
};

input TRAIDE_PRICE_TYPE InpPriceType = 4; //Price Input 
input int InpFastPeriod = 12;        // Fast Period
input int InpSlowPeriod = 26;        // Slow Period
input TRAIDE_MA_TYPE InpMAType = 0;  // Moving Average Type

#property indicator_buffers 1
#property indicator_color1 Orange
double indicatorBuffer0[];
uchar msgBuffer[200];
int periodoConver = 100;
int prev_calculated_mod=0;

#import "TRAIDE.dll"
    int TRAIDE_INIT(void);
    int TRAIDE_APO(int startIdx, int endIdx, const double &inReal[], int optInFastPeriod, int optInSlowPeriod, int optInMAType, int &outBegIdx, int &outNBElement, double &outReal[], char &msgBuffer[]);
#import

int scale;

int OnInit() {
     if (InpFastPeriod < 2 || InpSlowPeriod < 2) {
        return INIT_FAILED;
    }
 
    IndicatorShortName("APO(" + string(InpFastPeriod) + "," + string(InpSlowPeriod) + "," + EnumToString(InpMAType) + ")");
    IndicatorDigits(Digits);

    SetIndexLabel(0, "APO");
    SetIndexBuffer(0, indicatorBuffer0);
    SetIndexStyle (0, DRAW_LINE);
    ArraySetAsSeries(indicatorBuffer0, false); 

    //scale = (StringCompare(Symbol(), "USDJPY") == 0) ? 100 : 10000;
    scale=1;

    if(InpFastPeriod>InpSlowPeriod)
        periodoConver =  InpFastPeriod * 10;
    else
        periodoConver =  InpSlowPeriod * 10;

    TRAIDE_INIT();
    return INIT_SUCCEEDED;
}

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]){
     if (rates_total != prev_calculated) {        
        
        double inReal[];
        ArrayResize(inReal, rates_total);

        if(InpPriceType == OPEN_PRICE){
            ArraySetAsSeries(open, false);
            ArrayCopy(inReal, open);     
        }else if(InpPriceType == HIGH_PRICE){
           ArraySetAsSeries(high, false);
           ArrayCopy(inReal, high);     
        }else if(InpPriceType == LOW_PRICE){
           ArraySetAsSeries(low, false);
           ArrayCopy(inReal, low);     
        }else{
           ArraySetAsSeries(close, false);
           ArrayCopy(inReal, close);
        }

        int outBegIdx;
        int outNBElement;
        double outReal[];
        ArrayResize(outReal, rates_total);
   
        if(prev_calculated > periodoConver)
            prev_calculated_mod=prev_calculated-periodoConver;
        else
            prev_calculated_mod=0;

        if (TRAIDE_APO(prev_calculated_mod, rates_total - 1, inReal, InpFastPeriod, InpSlowPeriod, InpMAType, outBegIdx, outNBElement, outReal, msgBuffer) != 0) {
            Print(CharArrayToString(msgBuffer));
        }

        for (int i = 0; i < outNBElement; i++) { 
            indicatorBuffer0[outBegIdx + i] = outReal[i] * scale;
        }
    }
 
    return rates_total;
}
