/* ALPHADVISOR-Indicators - MT4 Custom Indicator
 * Copyright (C) 2017 ALPHADVISOR.
 * 
 *  This file is part of ALPHADVISOR-Indicators.
 *
 *  ALPHADVISOR-Indicators is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  ALPHADVISOR-Indicators is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ALPHADVISOR-Indicators.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  This software is an improved version based on:
 *  TRAIDE-Indicators - MT4 Custom Indicator
 *  Copyright (C) 2015 Inovance, Inc. http://inovancetech.com
 */
#property copyright "Copyright 2017, ALPHADVISOR"
#property link      "http://alphadvisor.com"
#property version   "2.1.0"
#property strict
#property indicator_separate_window

input int InpPeriod = 14;  // Period

#property indicator_buffers 2
#property indicator_color1 Orange
#property indicator_color2 Blue
double indicatorBuffer0[];
double indicatorBuffer1[];
uchar msgBuffer[200];
int periodoConver = 100;
int prev_calculated_mod=0;

#import "TRAIDE.dll"
    int TRAIDE_INIT(void);
    int TRAIDE_AROON(int startIdx, int endIdx, const double &inHigh[], const double &inLow[], int optInTimePeriod, int &outBegIdx, int &outNBElement, double &outAroonDown[], double &outAroonUp[], char &msgBuffer[]);
#import

int OnInit() {
     if (InpPeriod < 2) {
        return INIT_FAILED;
    }

    IndicatorShortName("AROON(" + string(InpPeriod) + ")");
    IndicatorDigits(Digits);

    SetIndexLabel(0, "AROON_DOWN");
    SetIndexBuffer(0, indicatorBuffer0);
    SetIndexStyle (0, DRAW_LINE);
    ArraySetAsSeries(indicatorBuffer0, false); 
    
    SetIndexLabel(1, "AROON_UP");
    SetIndexBuffer(1, indicatorBuffer1);
    SetIndexStyle (1, DRAW_LINE);
    ArraySetAsSeries(indicatorBuffer1, false); 

    periodoConver = InpPeriod * 10;

    TRAIDE_INIT();
    return INIT_SUCCEEDED;
}
  
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]){
     if (rates_total != prev_calculated) {    
        ArraySetAsSeries(high, false);
        ArraySetAsSeries(low, false);
 
        double inHigh[], inLow[];
        ArrayResize(inHigh, rates_total);
        ArrayResize(inLow, rates_total);
        ArrayCopy(inHigh, high);
        ArrayCopy(inLow, low);
        int outBegIdx, outNBElement;
        double outAroonDown[], outAroonUp[];
        ArrayResize(outAroonDown, rates_total); 
        ArrayResize(outAroonUp, rates_total); 

        if(prev_calculated > periodoConver)
            prev_calculated_mod=prev_calculated-periodoConver;
        else
            prev_calculated_mod=0;
   
        if (TRAIDE_AROON(prev_calculated_mod, rates_total - 1, inHigh, inLow, InpPeriod, outBegIdx, outNBElement, outAroonDown, outAroonUp, msgBuffer) != 0) {
            Print(CharArrayToString(msgBuffer));
        }

        for (int i = 0; i < outNBElement; i++) { 
            indicatorBuffer0[outBegIdx + i] = outAroonUp[i];//Descruzado
            indicatorBuffer1[outBegIdx + i] = outAroonDown[i];//Descruzado
        }        
    }
 
    return rates_total;
}
