/* ALPHADVISOR-Indicators - MT4 Custom Indicator
 * Copyright (C) 2017 ALPHADVISOR.
 * 
 *  This file is part of ALPHADVISOR-Indicators.
 *
 *  ALPHADVISOR-Indicators is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  ALPHADVISOR-Indicators is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ALPHADVISOR-Indicators.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  This software is an improved version based on:
 *  TRAIDE-Indicators - MT4 Custom Indicator
 *  Copyright (C) 2015 Inovance, Inc. http://inovancetech.com
 */
#property copyright "Copyright 2017, ALPHADVISOR"
#property link      "http://alphadvisor.com"
#property version   "2.1.0"
#property strict
#property indicator_separate_window

enum TRAIDE_MA_TYPE {
    SMA,   // Simple Moving Average (SMA)
    EMA,   // Exponential Moving Average (EMA)
    WMA,   // Weighted Moving Average (EMA)
    DEMA,  // Double Exponential Moving Average (DEMA)
    TEMA,  // Triple Exponential Moving Average (TEMA)
    TRIMA, // Triangular Moving Average (TRIMA)
    KAMA,  // Kaufman Moving Average (KAMA)
    MAMA,  // MESA Adaptative Moving Average (MAMA)
    T3     // Triple Exponential Moving Average (T3)
};

input int InpFastKPeriod = 5;              // Fast K Period
input int InpSlowKPeriod = 3;              // Slow K Period
input TRAIDE_MA_TYPE InpSlowKMAType = 0;   // Slow K Moving Average Type
input int InpSlowDPeriod = 3;              // Slow D Period
input TRAIDE_MA_TYPE InpSlowDMAType = 0;   // Slow D Moving Average Type

#property indicator_buffers 2
#property indicator_color1 Blue
#property indicator_color2 Red
double indicatorBuffer0[];
double indicatorBuffer1[];
uchar msgBuffer[200];
int periodoConver = 100;
int prev_calculated_mod=0;

#import "TRAIDE.dll"
    int TRAIDE_INIT(void);
    int TRAIDE_STOCH(int startIdx, int endIdx, 
        const double &inHigh[], const double &inLow[], const double &inClose[],
        int optInFastKPeriod, int optInSlowKPeriod, int optSlowKMAType, int optInSlowDPeriod, int optInSlowDMAType,        
        int &outBegIdx, int &outNBElement, double &outSlowK[], double &outSlowD[], char &msgBuffer[]);
#import

int OnInit() {
     if (InpFastKPeriod < 1 || InpSlowKPeriod < 1 || InpSlowDPeriod < 1) {
        return INIT_FAILED;
    }

    IndicatorShortName("STOCH(" + string(InpFastKPeriod) + "," 
        + string(InpSlowKPeriod) + "," + EnumToString(InpSlowKMAType) + ","
        + string(InpSlowDPeriod) + "," + EnumToString(InpSlowDMAType) + ")");
    IndicatorDigits(Digits);
    
    SetIndexLabel(0, "STOCH_SLOWK");
    SetIndexBuffer(0, indicatorBuffer0);
    SetIndexStyle (0, DRAW_LINE);
    ArraySetAsSeries(indicatorBuffer0, false); 

    SetIndexLabel(1, "STOCH_SLOWD");
    SetIndexBuffer(1, indicatorBuffer1);
    SetIndexStyle (1, DRAW_LINE);
    ArraySetAsSeries(indicatorBuffer1, false); 

    if(InpFastKPeriod>InpSlowKPeriod && InpFastKPeriod>InpSlowDPeriod)
        periodoConver =  InpFastKPeriod * 10;
    if(InpSlowKPeriod>InpFastKPeriod && InpSlowKPeriod>InpSlowDPeriod)
        periodoConver =  InpSlowKPeriod * 10;
    else
        periodoConver =  InpSlowDPeriod * 10;

    TRAIDE_INIT();
    return INIT_SUCCEEDED;
}
  
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]){     
     if (rates_total != prev_calculated) {    
        ArraySetAsSeries(high, false);
        ArraySetAsSeries(low, false);
        ArraySetAsSeries(close, false);

        double inHigh[], inLow[], inClose[];
        ArrayResize(inHigh, rates_total);
        ArrayCopy(inHigh, high);
        ArrayResize(inLow, rates_total);
        ArrayCopy(inLow, low);
        ArrayResize(inClose, rates_total);
        ArrayCopy(inClose, close);
        int outBegIdx;   
        int outNBElement;
        double outSlowK[];
        ArrayResize(outSlowK, rates_total); 
        double outSlowD[];
        ArrayResize(outSlowD, rates_total); 
   
        if(prev_calculated > periodoConver)
            prev_calculated_mod=prev_calculated-periodoConver;
        else
            prev_calculated_mod=0;

        if (TRAIDE_STOCH(prev_calculated_mod, rates_total - 1, 
            inHigh, inLow, inClose, 
            InpFastKPeriod, 
            InpSlowKPeriod, InpSlowKMAType, 
            InpSlowDPeriod, InpSlowDMAType, 
            outBegIdx, outNBElement, 
            outSlowK, outSlowD, msgBuffer) != 0) {
            Print(CharArrayToString(msgBuffer));
        }

         for (int i = 0; i < outNBElement; i++) { 
            indicatorBuffer0[outBegIdx + i] = outSlowD[i];//Descruzado
            indicatorBuffer1[outBegIdx + i] = outSlowK[i];//Descruzado
         }
    }
 
    return rates_total;
}
