/* ALPHADVISOR-Indicators - MT4 Custom Indicator
 * Copyright (C) 2017 ALPHADVISOR.
 * 
 *  This file is part of ALPHADVISOR-Indicators.
 *
 *  ALPHADVISOR-Indicators is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  ALPHADVISOR-Indicators is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ALPHADVISOR-Indicators.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  This software is an improved version based on:
 *  TRAIDE-Indicators - MT4 Custom Indicator
 *  Copyright (C) 2015 Inovance, Inc. http://inovancetech.com
 */
#property copyright "Copyright 2017, ALPHADVISOR"
#property link      "http://alphadvisor.com"
#property version   "2.1.0"
#property strict
#property indicator_chart_window

enum TRAIDE_MA_TYPE {
    SMA,   // Simple Moving Average (SMA)
    EMA,   // Exponential Moving Average (EMA)
    WMA,   // Weighted Moving Average (EMA)
    DEMA,  // Double Exponential Moving Average (DEMA)
    TEMA,  // Triple Exponential Moving Average (TEMA)
    TRIMA, // Triangular Moving Average (TRIMA)
    KAMA,  // Kaufman Moving Average (KAMA)
    MAMA,  // MESA Adaptative Moving Average (MAMA)
    T3     // Triple Exponential Moving Average (T3)
};
enum TRAIDE_PRICE_TYPE {
    OPEN_PRICE=1,
    HIGH_PRICE=2,
    LOW_PRICE=3,
    CLOSE_PRICE=4,
};

input TRAIDE_PRICE_TYPE InpPriceType = 4; //Price Input 
input int InpPeriod = 5;            // Period
input double InpNbDevUp = 2.0;      // Upper Band Multiplier
input double InpNbDevDown = 2.0;    // Lower Band Multiplier
input TRAIDE_MA_TYPE InpMAType = 0; // Moving Average Type

#property indicator_buffers 3
#property indicator_color1 Blue
#property indicator_color2 Orange
#property indicator_color3 Blue
double indicatorBuffer0[];
double indicatorBuffer1[];
double indicatorBuffer2[];
uchar msgBuffer[200];
int periodoConver = 100;
int prev_calculated_mod=0;

#import "TRAIDE.dll"
    int TRAIDE_INIT(void);
    int TRAIDE_BBANDS(int startIdx, int endIdx, const double &inReal[], 
        int optInTimePeriod, double optInNbDevUp, double optInNbDevDn, int optInMAType,    
        int &outBegIdx, int &outNBElement, 
        double &outRealUpperBand[], double &outRealMiddleBand[], double &outRealLowerBand[], char &msgBuffer[]);
#import

int OnInit() {
     if (InpPeriod < 2 || InpPeriod > 100000) {
        return INIT_FAILED;
    }
    
    IndicatorShortName("BBANDS(" + string(InpPeriod) + "," + string(InpNbDevUp) + "," + string(InpNbDevDown) 
        + "," + EnumToString(InpMAType) + ")");
    IndicatorDigits(Digits);
    
    SetIndexLabel(0, "BBAND_LOWER");
    SetIndexBuffer(0, indicatorBuffer0);
    SetIndexStyle (0, DRAW_LINE);
    ArraySetAsSeries(indicatorBuffer0, false); 

    SetIndexLabel(1, "BBAND_CENTER");
    SetIndexBuffer(1, indicatorBuffer1);
    SetIndexStyle (1, DRAW_LINE);
    ArraySetAsSeries(indicatorBuffer1, false); 

    SetIndexLabel(2, "BBAND_UPPER");
    SetIndexBuffer(2, indicatorBuffer2);
    SetIndexStyle (2, DRAW_LINE);
    ArraySetAsSeries(indicatorBuffer2, false); 

    periodoConver = InpPeriod * 10;

    TRAIDE_INIT();
    return INIT_SUCCEEDED;
}
  
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]){
     if (rates_total != prev_calculated) {    
     
        double inReal[];
        ArrayResize(inReal, rates_total);

        if(InpPriceType == OPEN_PRICE){
            ArraySetAsSeries(open, false);
            ArrayCopy(inReal, open);     
        }else if(InpPriceType == HIGH_PRICE){
           ArraySetAsSeries(high, false);
           ArrayCopy(inReal, high);     
        }else if(InpPriceType == LOW_PRICE){
           ArraySetAsSeries(low, false);
           ArrayCopy(inReal, low);     
        }else{
           ArraySetAsSeries(close, false);
           ArrayCopy(inReal, close);
        }

        int outBegIdx;   
        int outNBElement;
        double outRealUpperBand[];
        ArrayResize(outRealUpperBand, rates_total); 
        double outRealMiddleBand[];
        ArrayResize(outRealMiddleBand, rates_total); 
        double outRealLowerBand[];
        ArrayResize(outRealLowerBand, rates_total);
        
        if(prev_calculated > periodoConver)
            prev_calculated_mod=prev_calculated-periodoConver;
        else
            prev_calculated_mod=0;
   
        if (TRAIDE_BBANDS(prev_calculated_mod, rates_total - 1, inReal, 
            InpPeriod, InpNbDevUp, InpNbDevDown, InpMAType, 
            outBegIdx, outNBElement, outRealUpperBand, outRealMiddleBand, outRealLowerBand, msgBuffer) != 0) {
            Print(CharArrayToString(msgBuffer));
        }
         

        for (int i = 0; i < outNBElement; i++) { 
            indicatorBuffer0[outBegIdx + i] = outRealLowerBand[i];//Descruzados
            indicatorBuffer1[outBegIdx + i] = outRealMiddleBand[i];//Descruzados
            indicatorBuffer2[outBegIdx + i] = outRealUpperBand[i];//Descruzados
        }
    }
 
    return rates_total;
}
