/* ALPHADVISOR-Indicators - MT4 Custom Indicator
 * Copyright (C) 2017 ALPHADVISOR.
 * 
 *  This file is part of ALPHADVISOR-Indicators.
 *
 *  ALPHADVISOR-Indicators is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  ALPHADVISOR-Indicators is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ALPHADVISOR-Indicators.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  This software is an improved version based on:
 *  TRAIDE-Indicators - MT4 Custom Indicator
 *  Copyright (C) 2015 Inovance, Inc. http://inovancetech.com
 */
#property copyright "Copyright 2017, ALPHADVISOR"
#property link      "http://alphadvisor.com"
#property version   "2.1.0"
#property strict
#property indicator_chart_window

enum TRAIDE_PRICE_TYPE {
    OPEN_PRICE=1,
    HIGH_PRICE=2,
    LOW_PRICE=3,
    CLOSE_PRICE=4,
};

input TRAIDE_PRICE_TYPE InpPriceType = 4; //Price Input 
input double InpFastLimit = 0.5;  // Fast Limit
input double InpSlowLimit = 0.05; // Slow Limit

#property indicator_buffers 2
#property indicator_color1 Orange
#property indicator_color2 Blue
double indicatorBuffer0[];
double indicatorBuffer1[];
uchar msgBuffer[200];
int periodoConver = 100;
int prev_calculated_mod=0;

#import "TRAIDE.dll"
    int TRAIDE_INIT(void);
    int TRAIDE_MAMA(int startIdx, int endIdx, const double &inReal[], double optInFastLimit, double optInSlowLimit, int &outBegIdx, int &outNBElement, double &outMAMA[], double &outFAMA[], char &msgBuffer[]);
#import

int OnInit() {
    IndicatorShortName("MAMA(" + string(InpFastLimit) + "," + string(InpSlowLimit) + ")");
    IndicatorDigits(Digits);

    SetIndexLabel(0, "MAMA");
    SetIndexBuffer(0, indicatorBuffer0);
    SetIndexStyle (0, DRAW_LINE);
    ArraySetAsSeries(indicatorBuffer0, false); 

    SetIndexLabel(1, "FAMA");
    SetIndexBuffer(1, indicatorBuffer1);
    SetIndexStyle (1, DRAW_LINE);
    ArraySetAsSeries(indicatorBuffer1, false); 

    TRAIDE_INIT();
    return INIT_SUCCEEDED;
}
  
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]){
     if (rates_total != prev_calculated) {    

        double inReal[];
        ArrayResize(inReal, rates_total);

        if(InpPriceType == OPEN_PRICE){
            ArraySetAsSeries(open, false);
            ArrayCopy(inReal, open);     
        }else if(InpPriceType == HIGH_PRICE){
           ArraySetAsSeries(high, false);
           ArrayCopy(inReal, high);     
        }else if(InpPriceType == LOW_PRICE){
           ArraySetAsSeries(low, false);
           ArrayCopy(inReal, low);     
        }else{
           ArraySetAsSeries(close, false);
           ArrayCopy(inReal, close);
        }

        int outBegIdx;   
        int outNBElement;
        double outMAMA[];
        ArrayResize(outMAMA, rates_total); 
        double outFAMA[];
        ArrayResize(outFAMA, rates_total); 

        if(prev_calculated > periodoConver)
            prev_calculated_mod=prev_calculated-periodoConver;
        else
            prev_calculated_mod=0;
   
        if (TRAIDE_MAMA(prev_calculated_mod, rates_total - 1, inReal, InpFastLimit, InpSlowLimit, outBegIdx, outNBElement, outMAMA, outFAMA, msgBuffer) != 0) {
            Print(CharArrayToString(msgBuffer));
        }

        for (int i = 0; i < outNBElement; i++) {
            indicatorBuffer0[outBegIdx + i] = outFAMA[i];//Descruzado
            indicatorBuffer1[outBegIdx + i] = outMAMA[i];//Descruzado
        }
    }
 
    return rates_total;
}
